/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: DrawPosition.h,v 1.6 2003/09/05 05:02:04 yoshi Exp $ */ 
#ifndef DrawPosition_h
#define DrawPosition_h

#include <assert.h>

#include "Length.h"

class DrawPositionOperator
{
public:
	virtual ~DrawPositionOperator() {;}
	virtual int	start(int value, const Length &size) const = 0;
	virtual int	center(int value, const Length &size) const = 0;
	virtual int	end(int value, const Length &size) const = 0;
	virtual int	shift(int value, const Length &d) const = 0;
	virtual int	start(int value, int size) const = 0;
	virtual int	center(int value, int size) const = 0;
	virtual int	end(int value, int size) const = 0;
	virtual int	shift(int value, int d) const = 0;
};

class DrawPosition
{
private:
	DrawPositionOperator *getStart() const;
	DrawPositionOperator *getCenter() const;
	DrawPositionOperator *getEnd() const;

private:
	int			m_value;
	DrawPositionOperator	*m_op;

	void	set(int v, DrawPositionOperator *op) { m_value = v; m_op = op; }

public:
	DrawPosition() : m_value(0), m_op(0) {;}

	void	setStart(int v) { set(v, getStart()); }
	void	setCenter(int v) { set(v, getCenter()); }
	void	setEnd(int v) { set(v, getEnd()); }

	int	start(const Length &size) const { assert(m_op); return m_op->start(m_value, size); }
	int	center(const Length &size) const { assert(m_op); return m_op->center(m_value, size); }
	int	end(const Length &size) const { assert(m_op); return m_op->end(m_value, size); }
	int	shift(Length &d) { assert(m_op); return (m_value = m_op->shift(m_value, d)); }

	int	start(int size) const { assert(m_op); return m_op->start(m_value, size); }
	int	center(int size) const { assert(m_op); return m_op->center(m_value, size); }
	int	end(int size) const { assert(m_op); return m_op->end(m_value, size); }
	int	shift(int d) { assert(m_op); return (m_value = m_op->shift(m_value, d)); }

	int	innerStart(const Length &l) const { return start(l) + l.innerStart();	}
	int	innerEnd(const Length &l) const { return start(l) + l.innerEnd(); }

};

#endif  // DrawPosition_h
