/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: TextLineReadIteratorImpl.cpp,v 1.8 2003/12/12 12:49:04 yoshi Exp $ */
#include <string.h>
#include "TextLineReadIteratorImpl.h"
#include "CharReplacer.h"
#include "IOUtil.h"

#ifdef __sun__
#define SEEK_SET 0
#endif

const int INIT_LINE_INDEX = 0;


TextLineReadIteratorImpl::TextLineReadIteratorImpl()
	: m_buf(0), m_bufsize(0), m_replacer(0), m_line(INIT_LINE_INDEX), m_fp(0)
{
	;
}

TextLineReadIteratorImpl::~TextLineReadIteratorImpl()
{
	if (m_buf) { delete [] m_buf; }
}

char * TextLineReadIteratorImpl::
alloc(int size)
{
	if (m_buf) { delete [] m_buf; }
	m_bufsize = size;
	return (m_buf = new char [m_bufsize]);
}

const int BUF_UNIT_SIZE = 256;

bool TextLineReadIteratorImpl::next()
{
	if (!m_buf) { alloc(BUF_UNIT_SIZE); }

	while (!fgets(m_buf, m_bufsize, m_fp)) {
		if (feof(m_fp)) { return false; }
		alloc(m_bufsize+BUF_UNIT_SIZE);
	}
	if (IOUtil::chomp(m_buf)) { m_line++; }
	if (m_replacer) { replace(); }
	return true;
}


char	*TextLineReadIteratorImpl::replace()
{
	int  newsize;
	char *newbuf = m_replacer->exec(m_buf, newsize);
	if (newbuf != m_buf) {
		delete [] m_buf;
		m_buf = newbuf; m_bufsize = newsize;
	}
	return m_buf;
}

char * TextLineReadIteratorImpl::element()
{
	return m_buf;
}


void	TextLineReadIteratorImpl::goToStart()
{
	if (m_fp) {
		fseek(m_fp, 0, SEEK_SET);
		m_line = INIT_LINE_INDEX;
	}
}

char * TextLineReadIteratorImpl::element(int i)
{
	if (i <= m_line) { goToStart(); }
	while (m_line < i) {
		if (!next()) { return 0; }
	}
	return m_buf;
}

char * TextLineReadIteratorImpl::releaseElement()
{
	char *p = m_buf;
	m_buf = 0;
	return p;
}

void TextLineReadIteratorImpl::set(CharReplacer *p)
{
	m_replacer = p;
}
