/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: Math2DContextCreator.cpp,v 1.2 2003/09/16 09:11:47 yoshi Exp $ */ 
#include "Math2DContextCreator.h"
#include "DrawContext.h"
#include "EquationSizeManager.h"
#include "FontManager.h"
#include "HilightSwitch.h"
#include "EquationColor.h"


Math2DContextCreator::Math2DContextCreator(long &fs, long &sn, int &enum_)
	: Math2DContextCreatorBase(), m_fontSize(fs), m_startNumber(sn), m_enum(enum_),
	  m_hSwitch(0), m_color(0), m_sizeMgr(0), m_fontMgr(0)
{
	;
}

#define FREE(P) if (P) { delete P; P = 0; }
	
void Math2DContextCreator::free_()
{
	FREE(m_hSwitch);
	FREE(m_color);
	FREE(m_sizeMgr);
	FREE(m_fontMgr);
}


DrawContext *Math2DContextCreator::create(EquationRegister *reg, EquationDictionaryBase *dic)
{
	EquationSizeManager *eqSz = sizeMgr();
	HilightSwitch *hs = hilightSwitch();
	m_fontMgr->initFont();
	return new DrawContext(eqSz, m_fontMgr, hs, dic, reg, startNumber());

}

HilightSwitch *	Math2DContextCreator::hilightSwitch()
{
	if (!m_color) { m_color = new EquationColor; }
	if (!m_hSwitch) { m_hSwitch = new HilightSwitch(*m_color); }
	return m_hSwitch;
}

void Math2DContextCreator::color(unsigned normal_, unsigned hilight_, unsigned link_)
{
	FREE(m_color);
	FREE(m_hSwitch);
	m_color = new EquationColor(normal_, hilight_, link_);
	m_hSwitch = new HilightSwitch(*m_color);
}

EquationSizeManager * Math2DContextCreator::sizeMgr()
{
	if (!m_sizeMgr) { m_sizeMgr = new EquationSizeManager(fontSize()); }
	if (!m_fontMgr) { m_fontMgr = new FontManager; }
	m_fontMgr->set(m_sizeMgr);
	return m_sizeMgr;
}

FontManager& Math2DContextCreator::fontManager()
{
	if (!m_fontMgr) {
		m_fontMgr = new FontManager;
	}
	return *m_fontMgr;
}

