/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: BackSlashStartSymbolParser.cpp,v 1.32 2004/04/22 08:16:17 yoshi Exp $ */ 
#include <string.h>
#include "ParserException.h"
#include "BackSlashStartSymbolParser.h"
#include "EquationParseCmpnt.h"
#include "QuotationEliminator.h"
#include "BracketPairParser.h"
#include "BracketPairParserManager.h"
#include "EquationParserUtil.h"

bool BackSlashStartSymbolParser::match(char *str, int head)
{
	const char c = str[0];
	return (c == '\\')? true: false;
}

class EquationGenerator
{
public:
	virtual ~EquationGenerator() {;}
	virtual Equation *create() = 0;
	virtual bool parseContent() = 0;
	virtual void setContent(Equation *) = 0;
};

class DescriptionMatcher
{
public:
	virtual ~DescriptionMatcher() {;}
	virtual EquationGenerator *generator() = 0;
};

static EquationGenerator *match(char *str, int &len);

class ContentBracketMatcher : public QuotationEliminator::Matcher
{
private:
	BracketPairParser *m_pair;
private:
	char *	match(char *str, int head)
	{
		return m_pair->end(str, head);
	}
public:
	ContentBracketMatcher(BracketPairParser *p)
		: QuotationEliminator::Matcher(), m_pair(p) {;}
};

class ContentEquationParser : public EquationChildParseParser
{
private:
	Equation *parse(ScriptBlockIterator &itr, ParseRange *target, ParseRange *parsed);
};

Equation *ContentEquationParser::parse(ScriptBlockIterator &itr, ParseRange *target, ParseRange *parsed)
{
	itr.skipBlankRight();
	int	sx = itr.currx();
	char *buf = itr.lineString();

	Equation *eq = 0;
	BracketPairParser *pair_parser = BracketPairParserManager::startBracketCreate(buf);
	if (pair_parser) {

		// ʂ̒̋Lqp[X
		ContentBracketMatcher matcher(pair_parser);
		QuotationEliminator elim; elim.set(&matcher);
		char *p = elim.find(buf+1);

		delete pair_parser;

		int	ex = sx + p - buf;
		try {
			ParseRange *range = new ParseRange(sx+1, target->top(), ex-1, target->bottom());
			EquationParseCmpnt cmpnt(itr.block(), range);
			cmpnt.baseline(itr.curry());
			EquationRangeParserBase *rparser = createRangeParser();
			eq = rparser->parse(&cmpnt);
			delete rparser;
		} catch (ParseRange::OutOfRange) {;}

		if (parsed) { parsed->set(sx, target->top(), ex, target->bottom()); }
	} else {
		try { 
			ParseRange	*range = new ParseRange(sx, target->top(),
								target->right(), target->bottom());
			EquationParseCmpnt cmpnt(itr.block(), range);
			cmpnt.baseline(itr.curry());
			cmpnt.single(true); // E̍ŏ̋Lqp[X
			EquationRangeParserBase * parser = createRangeParser();
			eq = parser->parse(&cmpnt);
			if (parsed) {
				ParseRange &content_parsed = parser->parsed();
				parsed->set(sx, target->top(), content_parsed.right(), target->bottom());
			}
			delete parser;
		} catch (ParseRange::OutOfRange) {;}
	}

	return eq;
}


const char BACKSLASH = '\\';

#define BAD_DESCRIPTION_STR "bad description"

Equation *BackSlashStartSymbolParser::
parse(ScriptBlockIterator &itr, ParseRange *target, ParseRange *parsed)
{
	itr.skipBlankRight();
	int	sx = itr.currx();
	char *buf = itr.lineString();
	assert(*buf == BACKSLASH);
	int	len;
	EquationGenerator *gen = ::match(buf+1, len);
	if (!gen) {
		DTHROW_PE_MESSAGE(ParserIllegalDescriptionException, itr.totalLineIndex(),
			       (BAD_DESCRIPTION_STR " %s\n", buf));
	}

	int	ex = sx + len;
	Equation *eq = gen->create();
	if (gen->parseContent()) {
		ParseRange ctarget = *target;
		ParseRange cparsed;
		ContentEquationParser cparser;
		cparser.set(m_rangeParserCreator);
		EquationParser &parser = cparser;
		Equation *content = parser.parseBaseline(itr.block(), &ctarget, itr.curry(), ex+1,  &cparsed);
		if (content) { gen->setContent(content); }
		ex = cparsed.right();
	}
	delete gen;

	if (parsed) { parsed->set(target->left(), target->top(),
				  ex, target->bottom()); }
	return eq;
}

/* ######################################################################## 

          Matcher, EquationGenerator p ev[g&}N

  ######################################################################## */


#define DECLEAR_STRING_MATCHER(MATCHER, KEY, GENERATOR) \
class MATCHER : public DescriptionMatcher \
{\
public:\
	static EquationGenerator * match(char *, int &);\
};\
EquationGenerator *MATCHER::match(char *str, int &len)  {\
	len = strlen(KEY); \
	return (EquationParserUtil::matchString(str, KEY))? new GENERATOR(): 0; \
}

template<class EQUATION>
class EquationGeneratorTmp : public EquationGenerator
{
protected:
	EQUATION	*m_eq;
private:
	Equation *create() { return (m_eq = new EQUATION); }
public:
	EquationGeneratorTmp() : EquationGenerator(), m_eq(0) {;}
};

template<class EQUATION>
class ParseContentEquationGenerator : public EquationGeneratorTmp<EQUATION>
{
private:
	bool parseContent() { return true; }
protected:
	void setContent(Equation *content) { m_eq->content(content); }
};

template<class EQUATION>
class NotContentParseEquationGenerator : public EquationGenerator
{
private:
	Equation *create() { return new EQUATION; }
	bool parseContent() { return false; }
	void setContent(Equation *) { assert(0); }
};


/* ######################################################################## 

          ʂ

  ######################################################################## */

// -- backslash start --
#define HAT_STR "hat"
#define TILDE_STR "tilde"
#define IF_STR "if"
#define BF_STR "bf"
#define NONUMBER_STR "nonumber"
// -- backslash end --

//	Condition 
#include "ConditionEquation.h"
#include "BracketEquation.h"
class ConditionEquationGenerator : public ParseContentEquationGenerator<ConditionEquation> {};

DECLEAR_STRING_MATCHER(ConditionEquationMatcher, IF_STR, ConditionEquationGenerator);

//	Hat
#include "HatEquation.h"
class HatEquationGenerator : public ParseContentEquationGenerator<HatEquation> {};
DECLEAR_STRING_MATCHER(HatEquationMatcher, HAT_STR, HatEquationGenerator);

//	Tilde
#include "TildeEquation.h"
class TildeEquationGenerator : public ParseContentEquationGenerator<TildeEquation> {};
DECLEAR_STRING_MATCHER(TildeEquationMatcher, TILDE_STR, TildeEquationGenerator);

// bf
#include "VectorParamEquation.h"
class VectorParamEquationGenerator : public ParseContentEquationGenerator<VectorParamEquation> {};
DECLEAR_STRING_MATCHER(VectorParamEquationMatcher, BF_STR, VectorParamEquationGenerator);


#include "ContentEquationImpl.h"

#define DEF_EQUATION(EQ, STR, TEX_DESC, TEX_OK, WINDRAW_OK) \
	class EQ##Generator : public ParseContentEquationGenerator<EQ> {}; \
	DECLEAR_STRING_MATCHER(EQ##Matcher, STR, EQ##Generator)
#include "ContentEqs.h"
#undef DEF_EQUATION


/* ######################################################################## 

          ʂȂ

  ######################################################################## */

// -- backslash start --
#define NONUMBER_STR "nonumber"
// -- backslash end --

#include "NoNumberEquation.h"
class NoNumberEquationGenerator : public NotContentParseEquationGenerator<NoNumberEquation> {};
DECLEAR_STRING_MATCHER(NoNumberEquationMatcher, NONUMBER_STR, NoNumberEquationGenerator);


#include "BackSlashEquationImpl.h"

#define DECLARE_MATCHER_NC(EQ) \
typedef NotContentParseEquationGenerator<EQ> EQ##Generator; \
DECLEAR_STRING_MATCHER(EQ##Matcher, EQ##_STR, EQ##Generator)

#define DEF_EQUATION(EQ, STR, TEX_DESC, TEX, WIN) \
typedef NotContentParseEquationGenerator<EQ> EQ##Generator; \
DECLEAR_STRING_MATCHER(EQ##Matcher, STR, EQ##Generator)

#include "BackSlashSymbols.h"

#undef DEF_EQUATION

/* ######################################################################## 

          MV

  ######################################################################## */


#include "GreekCharacter.h"
template<GreekCharacterType TYPE>
class GreekCharacterGenerator : public EquationGenerator
{
private:
	GreekCharacter::Attribute	m_attr;
private:
	Equation *create() { return new GreekCharacter(TYPE, m_attr); }
	bool	parseContent() { return false; }
	void    setContent(Equation *) {;}
public:
	GreekCharacterGenerator(GreekCharacter::Attribute attr)
		: EquationGenerator(), m_attr(attr) {;}
};

template<GreekCharacterType TYPE>
class GreekCharacterMatcher : public DescriptionMatcher
{
public:
	static EquationGenerator* match(char *, int &len);
};

template<GreekCharacterType TYPE>
class VarGreekCharacterMatcher : public DescriptionMatcher
{
public:
	static EquationGenerator* match(char *, int &len);
};

#define MATCH_STRING(str, key) EquationParserUtil::matchString(str, key)

#define TO_LARGE(c) (c - 32)

#if 0 // for debug
typedef GreekCharacterMatcher<GREEK_CHARACTER_ALPHA> AlphaMatcher;
EquationGenerator *GreekCharacterMatcher<GREEK_CHARACTER_ALPHA>::
match(char *target, int &len)
{
	char *small = "alpha";
	char *head = small; 
	char *next = small + 1; 
        if (strncmp(target+1, next, (len = strlen(next))) != 0) { return 0; }
	len++; 
	if (*target == *head) {
		return new GreekCharacterGenerator<GREEK_CHARACTER_ALPHA>(GreekCharacter::SMALL);
	} 
	else if (*target == TO_LARGE(*head)) { 
		return new GreekCharacterGenerator<GREEK_CHARACTER_ALPHA>(GreekCharacter::LARGE);  
	} 
	return 0;
}

#else


// eMVNX̐錾match\bh
// Cve[Vs}N
#define IMPLEMENT_GREEK_MATCHER(SMALL_STR, TYPE, CLASSNAME) \
typedef GreekCharacterMatcher<TYPE> CLASSNAME##Matcher; \
EquationGenerator *GreekCharacterMatcher<TYPE>::match(char *target, int &len) { \
	char *head = SMALL_STR; \
	char *next = head + 1; \
	if (!MATCH_STRING(target+1, next)) { return 0; } \
	len = strlen(next) + 1; \
	if (*target == *head) { \
		return new GreekCharacterGenerator<TYPE>(GreekCharacter::SMALL); \
        } else if (*target == TO_LARGE(*head)) { \
		return new GreekCharacterGenerator<TYPE>(GreekCharacter::LARGE); \
	} \
	 return 0; \
}

#define IMPLEMENT_VAR_GREEK_MATCHER(STR, TYPE, CLASSNAME) \
typedef VarGreekCharacterMatcher<TYPE> CLASSNAME##Matcher; \
EquationGenerator *VarGreekCharacterMatcher<TYPE>::match(char *target, int &len) { \
	if (MATCH_STRING(target, STR)) { \
		len = strlen(STR); \
		return new GreekCharacterGenerator<TYPE>(GreekCharacter::VARIANT); \
	} else { return 0; } \
}
	
IMPLEMENT_GREEK_MATCHER("alpha", GREEK_CHARACTER_ALPHA, AlphaEquation);
IMPLEMENT_GREEK_MATCHER("beta", GREEK_CHARACTER_BETA, BetaEquation);
IMPLEMENT_GREEK_MATCHER("gamma", GREEK_CHARACTER_GAMMA, GammaEquation);
IMPLEMENT_GREEK_MATCHER("delta", GREEK_CHARACTER_DELTA, DeltaEquation);
IMPLEMENT_GREEK_MATCHER("epsilon", GREEK_CHARACTER_EPSILON, EpsilonEquation);
IMPLEMENT_GREEK_MATCHER("zeta", GREEK_CHARACTER_ZETA, ZetaEquation);
IMPLEMENT_GREEK_MATCHER("eta", GREEK_CHARACTER_ETA, EtaEquation);
IMPLEMENT_GREEK_MATCHER("theta", GREEK_CHARACTER_THETA, ThetaEquation);
IMPLEMENT_GREEK_MATCHER("iota", GREEK_CHARACTER_IOTA, IotaEquation);
IMPLEMENT_GREEK_MATCHER("kappa", GREEK_CHARACTER_KAPPA, KappaEquation);
IMPLEMENT_GREEK_MATCHER("lambda", GREEK_CHARACTER_LAMBDA, LambdaEquation);
IMPLEMENT_GREEK_MATCHER("mu", GREEK_CHARACTER_MU, MuEquation);
IMPLEMENT_GREEK_MATCHER("nu", GREEK_CHARACTER_NU, NuEquation);
IMPLEMENT_GREEK_MATCHER("xi", GREEK_CHARACTER_XI, XiEquation);
IMPLEMENT_GREEK_MATCHER("omicron", GREEK_CHARACTER_OMICRON, OmicronEquation);
IMPLEMENT_GREEK_MATCHER("pi", GREEK_CHARACTER_PI, PiEquation);
IMPLEMENT_GREEK_MATCHER("rho", GREEK_CHARACTER_RHO, RhoEquation);
IMPLEMENT_GREEK_MATCHER("sigma", GREEK_CHARACTER_SIGMA, SigmaEquation);
IMPLEMENT_GREEK_MATCHER("tau", GREEK_CHARACTER_TAU, TauEquation);
IMPLEMENT_GREEK_MATCHER("upsilon", GREEK_CHARACTER_UPSILON, UpsilonEquation);
IMPLEMENT_GREEK_MATCHER("phi", GREEK_CHARACTER_PHI, PhiEquation);
IMPLEMENT_GREEK_MATCHER("chi", GREEK_CHARACTER_CHI, ChiEquation);
IMPLEMENT_GREEK_MATCHER("psi", GREEK_CHARACTER_PSI, PsiEquation);
IMPLEMENT_GREEK_MATCHER("omega", GREEK_CHARACTER_OMEGA, OmegaEquation);

IMPLEMENT_VAR_GREEK_MATCHER("varepsilon", GREEK_CHARACTER_EPSILON, VarEpsilonEquation);
IMPLEMENT_VAR_GREEK_MATCHER("vartheta", GREEK_CHARACTER_THETA, VarThetaEquation);
IMPLEMENT_VAR_GREEK_MATCHER("varpi", GREEK_CHARACTER_PI, VarPiEquation);
IMPLEMENT_VAR_GREEK_MATCHER("varrho", GREEK_CHARACTER_RHO, VarRhoEquation);
IMPLEMENT_VAR_GREEK_MATCHER("varsigma", GREEK_CHARACTER_SIGMA, VarSigmaEquation);
IMPLEMENT_VAR_GREEK_MATCHER("varphi", GREEK_CHARACTER_PHI, VarPhiEquation);


#endif
#define EQUATION_MATCHING_START() EquationGenerator *gen; if (0) 

#define EQUATION_MATCHING(ClassName, STR, LEN) \
	else if ((gen = ClassName##Matcher::match(STR, LEN))) return gen

#define EQUATION_MATCHING4(ClassName, COND, STR, LEN) \
	else if ((gen = ClassName##Matcher::match(STR, LEN))) return gen


static EquationGenerator *match(char *str, int &len)
{
	EQUATION_MATCHING_START();

	EQUATION_MATCHING(AlphaEquation, str, len);
	EQUATION_MATCHING(BetaEquation, str, len);
	EQUATION_MATCHING(GammaEquation, str, len);
	EQUATION_MATCHING(DeltaEquation, str, len);
	EQUATION_MATCHING(EpsilonEquation, str, len);
	EQUATION_MATCHING(ZetaEquation, str, len);
	EQUATION_MATCHING(EtaEquation, str, len);
	EQUATION_MATCHING(ThetaEquation, str, len);
	EQUATION_MATCHING(IotaEquation, str, len);
	EQUATION_MATCHING(KappaEquation, str, len);
	EQUATION_MATCHING(LambdaEquation, str, len);
	EQUATION_MATCHING(MuEquation, str, len);
	EQUATION_MATCHING(NuEquation, str, len);
	EQUATION_MATCHING(XiEquation, str, len);
	EQUATION_MATCHING(OmicronEquation, str, len);
	EQUATION_MATCHING(PiEquation, str, len);
	EQUATION_MATCHING(RhoEquation, str, len);
	EQUATION_MATCHING(SigmaEquation, str, len);
	EQUATION_MATCHING(TauEquation, str, len);
	EQUATION_MATCHING(UpsilonEquation, str, len);
	EQUATION_MATCHING(PhiEquation, str, len);
	EQUATION_MATCHING(ChiEquation, str, len);
	EQUATION_MATCHING(PsiEquation, str, len);
	EQUATION_MATCHING(OmegaEquation, str, len);

	EQUATION_MATCHING(VarEpsilonEquation, str, len);
	EQUATION_MATCHING(VarThetaEquation, str, len);
	EQUATION_MATCHING(VarPiEquation, str, len);
	EQUATION_MATCHING(VarRhoEquation, str, len);
	EQUATION_MATCHING(VarSigmaEquation, str, len);
	EQUATION_MATCHING(VarPhiEquation, str, len);

	EQUATION_MATCHING(HatEquation, str, len);
	EQUATION_MATCHING(TildeEquation, str, len);
	EQUATION_MATCHING(VectorParamEquation, str, len);
	EQUATION_MATCHING(ConditionEquation, str, len);

	EQUATION_MATCHING(NoNumberEquation, str, len);
#define DEF_EQUATION(EQ, STR, TEX_DESC, TEX, WIN) EQUATION_MATCHING4(EQ, COND, str, len)
#include "BackSlashSymbols.h"
#include "ContentEqs.h"
#undef DEF_EQUATION

#if 0
	DTHROW_PE_MESSAGE(ParserIllegalDescriptionException,
		       (BAD_DESCRIPTION_STR " : \\%s\n", str));
#endif

	return 0;
}

