/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: EquationParserFactory.cpp,v 1.26 2004/02/19 13:43:51 yoshi Exp $ */ 
#include <ctype.h>
#include <string.h>
#include "EquationParserFactory.h"
#include "ScriptBlockRangeIterator.h"
#include "QuotationEliminator.h"
#include "StringUtil.h"

// parsers
#include "SummationEquationParser.h"
#include "ParamEquationParser.h"
#include "ComparativeOperatorEquationParser.h"
#include "BinominalOperatorEquationParser.h"
#include "BracketEquationParser.h"
#include "BackSlashStartSymbolParser.h"
#include "IntegralEquationParser.h"
#include "TaggedEquationParser.h"
#include "LinkEquationParser.h"
#include "MatrixEquationParser.h"
#include "FractionEquationParser.h"
#include "ProductEquationParser.h"
#include "NewLineParser.h"
#include "AlignEquationParser.h"
#include "DotEquationParser.h"
#include "SingleCharacterEquationParser.h"
#include "NotImplementedDescriptionParser.h"
#include "DefinitionEquationParser.h"
#include "ReferenceEquationParser.h"
#include "RawDescriptionParser.h"
#include "ExtendedRangeEquationParserImpl.h"
#include "NoNumberEquationParser.h"

#define PARSER_MATCHING_START() if (0)
#define PARSER_MATCHING(Name, str) else if (Name::match(str, 0)) parser = new Name

static char * findNewLine(char *str);
static char * findComparativeOperator(char *str);
static char * findBinominalOperator(char *str);
static char * findAlignEquation(char *str);
inline char *skipBlank(char *str) { return StringUtil::skip(str, SCRIPT_BLOCK_ELEMENT_BLANK); }


EquationParser *EquationParserFactory::createNewLine(ScriptBlockWalker *block, ParseRange *target, int base, int &startx)
{
	ScriptBlockRangeIterator itr(block, target);
	startx = target->left();
	itr.setPosition(startx, base);

	char *buf = itr.lineString();
	if (!buf) { return 0; }
	char *	head = skipBlank(buf);
	if (!*head) { return 0; }	// 󔒂݂̂ŋLqȂB
	char	*strFind;

	EquationParser *parser = 0;

	if (strFind = findNewLine(head)) {
		startx += (strFind - buf);
		parser = new NewLineParser;
	}
	if (parser) { parser->set(m_rangeParserCreator); }
	return parser;
}

EquationParser *EquationParserFactory::
create(ScriptBlockWalker *block, ParseRange *target, int base, int &startx)
{
	ScriptBlockRangeIterator itr(block, target);
	startx = target->left();
	itr.setPosition(startx, base);

	char *buf = itr.lineString();
	if (!buf) { return 0; }
	char *	head = skipBlank(buf);
	if (!*head) { return 0; }	// 󔒂݂̂ŋLqȂB
	char	*strFind;

	EquationParser *parser;

	if (0) {
		
	} else if (strFind = findAlignEquation(head)) {
		startx += (strFind - buf);
		parser =  new AlignEquationParser;
	} else if (DefinitionEquationParser::match(head, 0)) {
		parser = new DefinitionEquationParser;
	} else if (strFind = findComparativeOperator(head)) {
		startx += (strFind - buf);
		parser = new ComparativeOperatorEquationParser;
	} else if (strFind = findBinominalOperator(head)) {
		startx += (strFind - buf);
		parser =  new BinominalOperatorEquationParser;
	} else {

		PARSER_MATCHING_START();
		PARSER_MATCHING(NoNumberEquationParser, head);
		PARSER_MATCHING(SummationEquationParser, head);
		PARSER_MATCHING(IntegralEquationParser, head);
		PARSER_MATCHING(RawDescriptionParser, head); // bracket 
		PARSER_MATCHING(BracketEquationParser, head);
		PARSER_MATCHING(BackSlashStartSymbolParser, head);
		PARSER_MATCHING(LimitEquationParser, head); // Param 
		PARSER_MATCHING(MaxEquationParser, head);
		PARSER_MATCHING(MinEquationParser, head);
		PARSER_MATCHING(SupEquationParser, head);
		PARSER_MATCHING(InfEquationParser, head);
		PARSER_MATCHING(DotEquationParser, head); // Param 
		PARSER_MATCHING(ParamEquationParser, head);
		PARSER_MATCHING(TaggedEquationParser, head);
		PARSER_MATCHING(LinkEquationParser, head);
		PARSER_MATCHING(MatrixEquationParser, head);
		PARSER_MATCHING(FractionEquationParser, head);
		PARSER_MATCHING(ProductEquationParser, head);
		PARSER_MATCHING(SingleCharacterEquationParser, head);
		PARSER_MATCHING(ReferenceEquationParser, head);

		PARSER_MATCHING(NotImplementedDescriptionParser, head);
	}

	if (parser) { parser->set(m_rangeParserCreator); }
	return parser;
}

#define DEFINE_MATCHER(Name, ParserName) \
class Name : public QuotationEliminator::Matcher \
{\
private:\
	char *match(char *str, int head) \
	{ return (ParserName::match(str, head))? str+head: 0; } \
}

DEFINE_MATCHER(ComparativeOperatorMatcher, ComparativeOperatorEquationParser);
DEFINE_MATCHER(BinominalOperatorMatcher, BinominalOperatorEquationParser);
DEFINE_MATCHER(NewLineMatcher, NewLineParser);
DEFINE_MATCHER(AlignEquationMatcher, AlignEquationParser);

template <class MATCHER>
class DescriptionFinder
{
private:
	char *find(char *str)
	{
		MATCHER matcher;
		QuotationEliminator elim; elim.set(&matcher);
		return elim.find(str);
	}
public:
	char *operator()(char *str) { return find(str); }
};


typedef DescriptionFinder<ComparativeOperatorMatcher> ComparativeOperatorFinder;
typedef DescriptionFinder<BinominalOperatorMatcher> BinominalOperatorFinder;
typedef DescriptionFinder<NewLineMatcher> NewLineFinder;
typedef DescriptionFinder<AlignEquationMatcher> AlignEquationFinder;

static char * findComparativeOperator(char *str)
{
	ComparativeOperatorFinder find;
	return find(str);
}
static char * findBinominalOperator(char *str)
{
	BinominalOperatorFinder find;
	return find(str);
}

char * findNewLine(char *str)
{
	NewLineFinder find;
	return find(str);
}

static char * findAlignEquation(char *str)
{
	AlignEquationFinder find;
	return find(str);
}

void EquationParserFactory::free(EquationParser * &p)
{
	if (p) { delete p; p = 0; }
}
