/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: NamedEquationParser.cpp,v 1.6 2003/10/02 10:12:52 yoshi Exp $ */
#include "NamedEquationParser.h"
#include "NamedEquation.h"
#include "ParseRange.h"
#include "VarString.h"
#include "ParserException.h"
#include "EquationParseCmpnt.h"
#include "QuotationMatcher.h"
#include "QuotationEliminator.h"


#define CHAR_ATMARK '@'

static QuotationEliminator::Matcher *getNameQuatationMatcher(char *str);

#define TAG_END_STRING_SIZE 2

Equation * NamedEquationParser::
parse(ScriptBlockIterator &itr, ParseRange *target, ParseRange *parsed)
{
	int sx = itr.currx();
	char *buf = itr.lineString();
	if (!buf) { return 0; }
	QuotationEliminator::Matcher *matcher = getNameQuatationMatcher(buf);
	if (!matcher) { return 0; }
	VarString	name;
	char *content_start = extractName(buf, name);

	// ^O̓qɑΉ 021001
	QuotationEliminator elim; elim.set(matcher);
	char *find = elim.find(content_start);
	if (!find) { DTHROW_PE_MESSAGE(ParserIllegalDescriptionException,
				       itr.totalLineIndex(), 
				       ("not match tag\n"));
	}
	delete matcher;
	int end_tag_endx = (find-buf) + sx;

	NamedEquation *named = createEquation();
	named->name(name.c_str());
	try {
		const int startx = sx + (content_start-buf);
		const int endx = end_tag_endx-TAG_END_STRING_SIZE;
		ParseRange *range = new ParseRange(startx, target->top(),
						   endx, target->bottom());
		EquationParseCmpnt cmpnt(itr.block(), range);
		EquationRangeParserBase *rparser = createRangeParser();
		Equation *content = rparser->parse(&cmpnt);
		if (content) { named->content(content); }
		delete rparser;
	} catch (ParseRange::OutOfRange) {;}

	if (parsed) { parsed->set(target->left(), target->top(),
				  end_tag_endx, target->bottom()); }

	return named;
}

bool NamedEquationParser::match(char *str, int head)
{
	return (str[head] == CHAR_ATMARK)? true: false;
}

class NameQuotationMatcher : public QuotationEliminator::Matcher
{
private:
	char*	match(char *str, int head)
	{
		const char curr = str[head];
		char next = str[head+1];
		return ((curr == CHAR_ATMARK) && (next == CHAR_ATMARK))? str+head+1: 0;
	}
};

static QuotationEliminator::Matcher *getNameQuatationMatcher(char *str)
{
	return (NamedEquationParser::match(str, 0))?
		new NameQuotationMatcher: 0;
}
