/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: BracketEquationTexDrawer.cpp,v 1.7 2003/12/15 13:24:02 yoshi Exp $ */ 
#include "BracketEquationTexDrawer.h"
#include "BracketEquation.h"
#include "DrawContext.h"

class BracketSelector
{
protected:
	enum {
		VISIBLE_BRACE = 6,
		DOT,
		RIGHT_NUM,
	};
	enum { LEFT_NUM = RIGHT_NUM - 1, };
protected:
	BracketEquation	 &m_bra;
private:
	int	leftIndex();
	int	rightIndex();
protected:
	virtual char ** leftStrs() = 0;
	virtual char ** rightStrs() = 0;
public:
	BracketSelector(BracketEquation &bra) : m_bra(bra) {;}
	virtual ~BracketSelector() {;}
	char	*left();
	char	*right();
};

#define ARY_SIZE(ARY) (sizeof(ARY)/sizeof(ARY[0]))

#define LEFT(TOKEN) " \\left" TOKEN " "
#define RIGHT(TOKEN) " \\right" TOKEN " "

class NormalBracketSelector : public BracketSelector
{
public:
	NormalBracketSelector(BracketEquation &bra)
		: BracketSelector(bra) {;}
private:
	char ** leftStrs()
	{
		static char *strs[] = {
			LEFT("("), "{", LEFT("["),  
			"\\lfloor", "\\lceil", "\\langle",
			LEFT("\\{"),
		};
		assert(ARY_SIZE(strs) == LEFT_NUM);
		return strs;
	};
	char ** rightStrs()
	{
		static char *strs[] = {
			RIGHT(")"), "}", RIGHT("]"), 
			"\\rfloor", "\\rceil", "\\rangle",
			RIGHT("\\}"),
			RIGHT("."), };
		assert(ARY_SIZE(strs) == RIGHT_NUM);
		return strs;
	}
};

class MatrixBracketSelector : public BracketSelector
{
public:
	MatrixBracketSelector(BracketEquation &bra)
		: BracketSelector(bra) {;}
private:
	char ** leftStrs()
	{
		static char *strs[] = {
			"\\left(", 0, "\\left[",
			"\\lfloor", "\\lceil", "\\langle",
			"\\left\\{"};
		assert(ARY_SIZE(strs) == LEFT_NUM);
		return strs;
	};
	char ** rightStrs()
	{
		static char *strs[] = {
			"\\right)", 0, "\\right]",
			"\\rfloor", "\\rceil", "\\rangle",
			"\\right\\}", "\\right.", };
		assert(ARY_SIZE(strs) == RIGHT_NUM);
		return strs;
	}
};

class CasesBracketSelector : public BracketSelector
{
public:
	CasesBracketSelector(BracketEquation &bra)
		: BracketSelector(bra) {;}
private:
	char ** leftStrs() { return 0; }
	char ** rightStrs() { return 0; }
};

void BracketEquationTexDrawer::draw(Equation *eq, DrawContext *context)
{
	BracketEquation *bra = static_cast<BracketEquation*>(eq);
	DrawContext::Writer &writer = context->writer();

	Equation *content = bra->content();

	BracketSelector *str = 0;
	if (content && content->isMatrix()) {
		str = new MatrixBracketSelector(*bra);
	}
	const bool cases = eq->containCondition();
	if (cases) {
		assert(!str);
		str = new CasesBracketSelector(*bra);
	}
	if (!str) { str = new NormalBracketSelector(*bra); }

	char *p;
	p = str->left();
	if (p) { writer.write(p); writer.write(' ');}
		
	content->draw(context);

	p = str->right();
	if (p) { writer.write(p); writer.write(' ');}

	delete str;
}

int BracketSelector::leftIndex()
{
	int idx = m_bra.left();
	if ((idx == BracketEquation::SHAPE_BRACE) && m_bra.leftVisible()) {
		idx = VISIBLE_BRACE;
	}
	return idx;
}



int BracketSelector::rightIndex()
{
	int idx = m_bra.right();
	if (idx == BracketEquation::SHAPE_NOT_SET) {
		idx = DOT;
	} else 	if ((idx == BracketEquation::SHAPE_BRACE) && m_bra.rightVisible()) {
		// ʂ̂ƂEʂ'}' TeXŃG[
		idx = VISIBLE_BRACE;
	}

	if (idx == BracketEquation::SHAPE_BRACE && m_bra.leftVisible()) {
		idx = DOT;
	}
	return idx;
}

char * BracketSelector::left()
{
	char **strs = leftStrs();
	if (!strs) { return 0; }
	const int i = leftIndex();
	return strs[i];
}

char * BracketSelector::right()
{
	char **strs = rightStrs();
	if (!strs) { return 0; }
	const int i = rightIndex();
	return strs[i];
}
