/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: DrawContext.h,v 1.11 2004/05/06 05:10:28 yoshi Exp $ */ 
#ifndef DrawContext_h
#define DrawContext_h

#include <assert.h>
#include "TextWriter.h"
#include "TexType.h"

class EquationDictionaryBase;

class DrawContext
{
public:
	typedef TextWriter Writer;
	class LineCountWriter : public Writer
	{
	public:
		virtual int line() = 0;
		virtual void newline(const char *) = 0;
	};
private:
	enum State { STATE_NOT_SET   = 0x00,
		     ENUMERATE       = 0x01,
		     NAME            = 0x02,
		     NEWLINE         = 0x04,
		     RAW_DESCRIPTION = 0x08,
		     EQUATION_BLOCK  = 0x10,
		     NEWLINE_AS_TEX_COMMENT = 0x20,
	};

private:
	TextWriter	*m_writer;
	LineCountWriter	*m_counter;
	EquationDictionaryBase	*m_dictionary;

	int     m_state;
	TexType m_texType;

private:
	void	free_();
private:
	void setState(bool b, State s) {
		if (b) { m_state |= s; }
		else { m_state &= ~s; }
	}
	bool state(State s) { return (m_state & s)? true: false; }
	
public:
	DrawContext();
	DrawContext(const DrawContext &);
	~DrawContext() { free_(); }
	void	set(Writer *writer);
	void	set(EquationDictionaryBase *p) { m_dictionary = p; }

	void    enumerate(bool b) { setState(b, ENUMERATE); }
	bool    enumerate() { return state(ENUMERATE); }

	void    name(bool b) { setState(b, NAME); }
	bool    name() { return state(NAME); }

	void    newline(bool b) { setState(b, NEWLINE); }
	bool    newline() { return state(NEWLINE); }

	void   newlineAsTexComment(bool b) {
		setState(b, NEWLINE_AS_TEX_COMMENT);
	}
	bool    newlineAsTexComment() { return state(NEWLINE_AS_TEX_COMMENT); }

	void    rawDescription(bool b) { setState(b, RAW_DESCRIPTION); }
	bool    rawDescription() { return state(RAW_DESCRIPTION); }

	void    equationBlock(bool b) { setState(b, EQUATION_BLOCK);}
	bool    equationBlock() { return state(EQUATION_BLOCK); }
	
	TexType &texType() { return m_texType; }

	void	setup();

	Writer &writer() {
		assert(m_counter);
		return *m_counter;
	}
	EquationDictionaryBase &dictionary() {
		assert(m_dictionary);
		return *m_dictionary;
	}

	int	line() { return m_counter->line(); }
	void	writeNewline() {
		static char *comment = "%";
		char *head = newlineAsTexComment()? comment: 0;
		m_counter->newline(head);
	}
};

#endif // DrawContext_h
