/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: EquationEnumeratorEquationTexDrawer.cpp,v 1.7 2004/02/17 04:19:38 yoshi Exp $ */ 
#include "EquationEnumeratorEquationTexDrawer.h"
#include "EquationEnumeratorEquation.h"
#include "DrawContext.h"

class EquationBlock
{
public:
	virtual ~EquationBlock() {;}
	virtual char *tagname() = 0;
	void start(DrawContext::Writer &writer)
	{
		writer.write("\\begin{");
		writer.write(tagname());
		writer.write("}\n");
	}

	void end(DrawContext::Writer &writer)
	{
		writer.write("\n\\end{");
		writer.write(tagname());
		writer.write("}\n");
	}
};

class EquationEnumerationBlock : public EquationBlock
{
private:
	char *tagname()
	{
		static char *str = "eqnarray";
		return str;
	}
};

class EquationNoEnumerationBlock : public EquationBlock
{
private:
	char *tagname()
	{
		static char *str = "eqnarray*";
		return str;
	}
};

static bool outputBlock(Equation *eq, DrawContext *context)
{
	return (!eq->isDefinition() && context->equationBlock())? true: false;
}

static EquationBlock *getBlock(Equation *eq, DrawContext *context)
{
	static EquationEnumerationBlock enum_;
	static EquationNoEnumerationBlock noEnum;
	if (!outputBlock(eq, context)) { return 0;}
	return (context->enumerate())?
		static_cast<EquationBlock*>(&enum_):
		static_cast<EquationBlock*>(&noEnum);
}

void EquationEnumeratorEquationTexDrawer::draw(Equation *eq, DrawContext *context)
{
	EquationEnumeratorEquation *enum_ = static_cast<EquationEnumeratorEquation*>(eq);
	EquationEnumeratorEquation::Container &eqs = enum_->equations();
	const int n = eqs.size();
	if (n == 0) { return ; }
	DrawContext::Writer &writer = context->writer();

	EquationBlock *block = getBlock(enum_, context);
	if (block) { block->start(writer); }

	for (int i=0; i<n; i++) {
		Equation *e = eqs[i];
		e->draw(context);
		if (i != n-1) {
			writer.write(" \\\\\n");
		}
	     
	}	
	if (block) { block->end(writer); }
}
