/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: PHTex.cpp,v 1.12 2004/05/06 05:11:16 yoshi Exp $ */ 
#include "matex.h"
#include "PHTexEx.h"
#include "Exception.h"
#include "TextLineBufferContainer.h"
#include "TextLineBufferContainerIterator.h"
#include "TextBufferWriter.h"
#include "PHTexExec.h"
#include "PHTexParam.h"
#include "Version.h"
#include "Copyright.h"

#define BUFSIZE 8192

static char s_texbuf[BUFSIZE] = {0};
static char s_error[BUFSIZE] = {0};

static int s_startLine = 0;

static bool pushLine(const char *line);
static void setOption(unsigned options);
static char *toTex();
static void freePHTexLib();

static bool s_firstUse = true;

static void printCopyright()
{
	fputs(getCopyright(), stderr);
}

void PHTexSetStartLine(int l)
{
	s_startLine = l;
}

#ifdef __cplusplus
extern "C" {
#endif
	int	MatexInit()
	{
		if (s_firstUse) {
			printCopyright();
			s_firstUse = false;
		}
		freePHTexLib();
		return 0;
	}
	void	MatexFree()
	{
		freePHTexLib();
	}

	int	MatexPushLine(const char *line)
	{
		return (pushLine(line))? 0: 1;
	}
	char *	MatexToTex(unsigned options)
	{
		setOption(options);
		return toTex();
	}

	char *	MatexGetLastError()
	{
		return (s_error[0])? s_error: 0;
	}
		
	const char *  MatexGetVersion()
	{
		return getVersion();
	}
	
#ifdef __cplusplus
}
#endif

#define INIT_BUF(BUF) (BUF[0] = 0)

#define FREE(p) if (p) { delete p; p = 0; }

class ErrorTextBufferWriter : public ErrorTextWriter
{
private:
	TextBufferWriter	m_writer;
public:
	ErrorTextBufferWriter(char *buf) : m_writer(buf) {;}

	void	write(const char *msg, int lnum)
	{
		if (lnum < 0) {
			m_writer.write(msg);
		} else {
			const int n = s_startLine + lnum;
			m_writer.writeWithFormat("%d : %s", n, msg);
		}
	}
};

class PHTexLib : public PHTexParam
{
private:
	typedef PHTexParam Super;
private:
	TextLineBufferContainer	m_lines;
	char	m_texBuffer[BUFSIZE];
	char	m_errorBuffer[BUFSIZE];
public:
	PHTexLib() : Super()
	{
		INIT_BUF(m_texBuffer);
		INIT_BUF(m_errorBuffer);
	}

	bool	pushLine(const char *line)
	{
		return m_lines.push(line);
	}
	char	*toTex()
	{
		TextBufferWriter	texWriter(m_texBuffer);
		Super::setOutputStream(&texWriter);

		TextLineBufferContainerIterator itr(m_lines);
		Super::setInputStream(&itr);

		PHTexExec exec(*this);
		ErrorTextBufferWriter	errorWriter(m_errorBuffer);
		exec.set(&errorWriter);

		if (!exec.draw()) { return 0; }
		return m_texBuffer;
	}
	char	*lastError()
	{
		return m_errorBuffer;
	}
};

static PHTexLib *s_phtex = 0;

static PHTexLib *getPHTexLib()
{
	if (!s_phtex) {
		s_phtex = new PHTexLib;
		INIT_BUF(s_texbuf);
		INIT_BUF(s_error);
	}
	return s_phtex;
}

static void freePHTexLib()
{
	FREE(s_phtex);
	s_startLine = 0;
}

static bool pushLine(const char *line)
{
	PHTexLib *phtex = getPHTexLib();
	return phtex->pushLine(line);
}

static char *toTex()
{
	char *ret = 0;
	PHTexLib *phtex = getPHTexLib();
	char *str = phtex->toTex();
	if (str) { 
		strcpy(s_texbuf, str);
		ret = s_texbuf;
	}
	char *err = phtex->lastError();
	if (err) { strcpy(s_error, phtex->lastError());}

	// 021113 yoshi 
	// toTexĂ΂閈PHTexLibCX^X蒼̂ŁA
	// Ă΂ꂽƂ̃ftHgl̐SzKvȂ
	freePHTexLib();
	return ret;
}

static void setOption(unsigned options)
{
	PHTexLib *phtex = getPHTexLib();
#define CHECK_OPTION(FLAG, PROC) if (options & FLAG) { phtex->PROC; }
	CHECK_OPTION(PHTEX_ENUMERATE_EQUATION, enumerate(true));
	CHECK_OPTION(PHTEX_OUTPUT_TAG_DESCRIPTION, tag(true));
	CHECK_OPTION(PHTEX_ELIMINATE_EMBEDDED_DESCRIPTION, rawDescription(false));
	//	CHECK_OPTION(PHTEX_ONE_LINE, newline(false));
	CHECK_OPTION(PHTEX_SAME_LINE_NUMBER, sameLineNumber(true));
	CHECK_OPTION(PHTEX_NO_EQUATION_BLOCK, equationBlock(false));
	CHECK_OPTION(PHTEX_DOCUMENT, document(true));
	CHECK_OPTION(PHTEX_ASCII_PLATEX, asciiPlatexMode());
	CHECK_OPTION(PHTEX_NTT_JLATEX, nttJlatexMode());
	CHECK_OPTION(PHTEX_NEWLINE_AS_TEX_COMMENT, newlineAsTexComment(true));
#undef CHECK_OPTION
}


