/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: PHTexArgsTokenizer.h,v 1.13 2004/05/06 05:10:51 yoshi Exp $ */ 
#ifndef PHTexArgsTokenizer_h
#define PHTexArgsTokenizer_h

#include <string.h>

#include "matex.h"
#include "PHTexEx.h"

class PHTexArgsTokenizer
{
public:
	class Exception {};
	class PrintInfoException : public Exception {};
	class PrintHelpException : public PrintInfoException {};
	class PrintVersionException : public PrintInfoException {};
	class PrintTexLikeDescriptin : public PrintInfoException {};
	class IllegalOptionException : public Exception {};
public:
	typedef void (PHTexArgsTokenizer::*OptionProc)(char *);

	class Option
	{
	public:
		OptionProc     proc;
		bool           needsNext;
		char		*msg;
	public:
		Option(OptionProc proc_, bool next_, char *msg_)
			: proc(proc_), needsNext(next_), msg(msg_) {;}
	};
 

	class SingleOption : public Option
	{
	public:
		typedef char Key;
	public:
		Key    key;
	public:
		SingleOption(Key key_, OptionProc proc_, bool next_, char *msg_)
			: Option(proc_, next_, msg_), key(key_) {;}
		bool match(Key k) { return (key == k)? true: false; }
	};

	class DoubleOption : public Option
	{
	public:
		typedef char *  Key;
	public:
		Key             key;
	public:
		DoubleOption(Key key_, OptionProc proc_, bool next_, char *msg)
			: Option(proc_, next_, msg), key(key_) {;}
		bool match(Key k) {
			return (strncmp(key, k, strlen(key)) == 0)? true: false;
		}
	};

private:
	unsigned	m_options;
	unsigned	m_negOptions;
	char		*m_infile;
	char		*m_outfile;
	bool		m_inputTex;
	bool		m_ignoreRC;
	bool		m_infoException;
	
	Option *	m_option;
	char		m_err[1024];

private:
	// OptionProc ^\bh
	void	inputFilename(char *p) { m_infile = p; }
	void	outputFilename(char *p) { m_outfile = p; }
	void	debugMode(char *) { m_options |= PHTEX_DOCUMENT; }
	void	help(char *)
	{
		if (m_infoException) { throw PrintHelpException(); }
	}
	void    version(char *)
	{
		if (m_infoException) { throw PrintVersionException(); }
	}
	void	texDesc(char *)
	{
		if (m_infoException) {throw PrintTexLikeDescriptin(); }
	}
	void	enumerate(char *) { m_options |= PHTEX_ENUMERATE_EQUATION; }
	void	notEnumerate(char *) { m_negOptions |= PHTEX_ENUMERATE_EQUATION;}
	void    oneline(char*) { m_options |= PHTEX_ONE_LINE; }
	void	tag(char*) { m_options |= PHTEX_OUTPUT_TAG_DESCRIPTION; }
	void	eliminateRawDescription(char*) {
		m_options |= PHTEX_ELIMINATE_EMBEDDED_DESCRIPTION;
	}
	void    nttJlatexMode(char *) { m_options |= PHTEX_NTT_JLATEX; }
	void    asciiPlatexMode(char *) { m_options |= PHTEX_ASCII_PLATEX; }
	void    sameLineNumber(char *) {
		oneline(0);
		m_options |= PHTEX_SAME_LINE_NUMBER;
	}
	void    noEquationBlock(char *) { m_options |= PHTEX_NO_EQUATION_BLOCK; }
	void	inputTex(char *) {
		m_inputTex = true;
		m_options |= PHTEX_NEWLINE_AS_TEX_COMMENT;
	}
	void	outputTex(char *) {;} // reserved for future use
	void	ignoreRC(char *) { m_ignoreRC = true; }
private:
	SingleOption *getSingleOptions(int &n);
	int	setSingleOption(char *arg);
	Option *findOption(SingleOption::Key);

	DoubleOption *getDoubleOptions(int &n);
	int     setDoubleOption(char *arg);
	Option *findOption(DoubleOption::Key);
	
	int	execOptionProc(char *);

public:
	PHTexArgsTokenizer(bool throwException = true);

	void	printHelp(const char *);

	int	tokenize(int argc, char **argv);
	int	tokenize(const char *);
	int	operator()(int argc, char **argv) { return tokenize(argc, argv); }
	int	operator()(const char *args) { return tokenize(args); }

	char	*infile() { return m_infile; }
	char	*outfile() { return m_outfile; }

	unsigned	options() const { return m_options; }
	unsigned	negativeOptions() const { return m_negOptions; }
	unsigned	applyNegativeOptions(unsigned v) const
	{
		return v & (~m_negOptions);
	}
	bool	inputTex() { return m_inputTex; }
	bool	ignoreRC() { return m_ignoreRC; }
	char	*lastErr() { return m_err; }
};

#endif // PHTexArgsTokenizer_h
