%{
/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: PHTexBlock.l,v 1.15 2004/05/06 08:13:56 yoshi Exp $ */ 
#include "PHTexBlockParseListener.h"
#include <stdio.h>
#include <string.h>
#include <assert.h>
#ifdef WIN32
#include <io.h>
#endif

#define CMD_2DIMEQ "%2dimeq"

static int s_ignore = 1;
static int s_line = 0;
static PHTexBlockParseListener::Error s_error = PHTexBlockParseListener::BPE_NOERROR; 

static PHTexBlockParseListener *s_listener = 0;
static void defaultBlock(char *name);
static void ignoreBlock(int n);
static void convertBlock(int n);
static void defaultOption(const char *);
static void blockOption(const char *);

static PHTexBlockParseListener::Error startBlock(char *name);
static PHTexBlockParseListener::Error endBlock(char *name);
static PHTexBlockParseListener::Error (*blockNameProc)(char *) = 0;

static char s_buf[1024] = {0};
static char s_blockname[1024] = {0};
static int s_bufidx = 0;
static void (*s_optionProc)(const char *) = 0;
static void pushString(char *);
static bool flushLine(bool ret = true);
static void chomp(char *);
%}

BLOCKNAME_CHAR [a-zA-Z0-9*]
OPTION_CHAR [a-zA-Z1]

%x BLOCKNAME
%x EQ_DEFAULT_BLOCK
%x EQ_DEFAULT_OPTION
%x EQ_IGNORE
%x EQ_IGNORE_NUM
%x EQ_BLOCK
%x EQ_BLOCK_PROP
%x EQ_BLOCK_OPTION

%%
<BLOCKNAME>"}" { BEGIN INITIAL; }
<BLOCKNAME>{BLOCKNAME_CHAR}+ {
	/* ubNRs[Ă*/
         strcpy(s_blockname, yytext);
}
"%2dimeq-default-block=" { pushString(yytext); BEGIN EQ_DEFAULT_BLOCK; }
<EQ_DEFAULT_BLOCK>{BLOCKNAME_CHAR}+ { pushString(yytext); defaultBlock(yytext); }
<EQ_DEFAULT_BLOCK>"\n"  {
	if (!flushLine()) {
		yyterminate();
	}
	BEGIN INITIAL;
}
<EQ_DEFAULT_BLOCK>"\r" ;
<EQ_DEFAULT_BLOCK>. { pushString(yytext); }

"%2dimeq-ignore" { s_ignore = 1; pushString(yytext); BEGIN EQ_IGNORE; }
<EQ_IGNORE>= { pushString(yytext); BEGIN EQ_IGNORE_NUM; }
<EQ_IGNORE>"\r" ;
<EQ_IGNORE>"\n" { ignoreBlock(1); flushLine(); BEGIN INITIAL; }
<EQ_IGNORE>. ;
<EQ_IGNORE_NUM>[0-9]+ { s_ignore = atoi(yytext); pushString(yytext);}
<EQ_IGNORE_NUM>"\n" {
	ignoreBlock(s_ignore);
	if (!flushLine()) { yyterminate(); }
	BEGIN INITIAL; }
<EQ_IGNORE_NUM>. ;

"%2dimeq-default-opt" { pushString(yytext);; s_optionProc = defaultOption; BEGIN EQ_BLOCK_OPTION; }
"%2dimeq-opt" { pushString(yytext); s_optionProc = blockOption; BEGIN EQ_BLOCK_OPTION; }
<EQ_BLOCK_OPTION>"\r" ;
<EQ_BLOCK_OPTION>=.*"\n" {
        chomp(yytext);
        pushString(yytext);
	flushLine();
	(*s_optionProc)(yytext+1);
        BEGIN INITIAL;
}
<EQ_BLOCK_OPTION>. pushString(yytext);

"%2dimeq" { pushString(yytext); BEGIN EQ_BLOCK; }
<EQ_BLOCK>= { pushString(yytext); BEGIN EQ_BLOCK_PROP; }
<EQ_BLOCK>"\r" ;
<EQ_BLOCK>"\n" { convertBlock(1); flushLine(); BEGIN INITIAL;}
<EQ_BLOCK>. { pushString(yytext); }
<EQ_BLOCK_PROP>[0-9]+ { convertBlock(atoi(yytext)); pushString(yytext); }
<EQ_BLOCK_PROP>"\r" ;
<EQ_BLOCK_PROP>"\n" { flushLine(); BEGIN INITIAL; }
<EQ_BLOCK_PROP>. { pushString(yytext); }

"%".*"\r"?"\n" {  if (strncmp(yytext, "%2dimeq", strlen("%2dimeq")) == 0) { REJECT; }
         else {
           chomp(yytext);
           pushString(yytext);
           bool b = flushLine();
           if (!b) { yyterminate(); }
         }
      }
"\\begin{" {
	s_error = PHTexBlockParseListener::BPE_NOERROR;
	blockNameProc = startBlock;
	BEGIN BLOCKNAME;
}
"\\end{"  { blockNameProc = endBlock; BEGIN BLOCKNAME; }
"\n" { if (!flushLine()) { yyterminate(); } }
"\r" ;
.  { s_buf[s_bufidx++] = yytext[0]; }
%%

int yyerror() { return 0; }
int yywrap() { return 1; }

static int s_level = 0;

static void init()
{
	s_level = 0;
        s_ignore = 1;
        s_bufidx = 0;
        s_line = 0;
	s_error = PHTexBlockParseListener::BPE_NOERROR;
}

static PHTexBlockParseListener::Error startBlock(char *name)
{
        s_level++;
        return s_listener->startBlock(name, s_level);
}

static PHTexBlockParseListener::Error endBlock(char *name)
{
	PHTexBlockParseListener::Error e = s_listener->endBlock(name, s_level);
	s_level--;
        return e;
}

static void defaultBlock(char *name)
{
	s_listener->defaultBlock(name);
}

static void defaultOption(const char *name)
{
	s_listener->defaultOption(name);
}

static void ignoreBlock(int n)
{
	s_listener->ignoreBlock(n);
}

static void convertBlock(int n)
{
	s_listener->convertBlock(n); 
}

static void blockOption(const char *str)
{
	s_listener->blockOption(str);
}
			    
static void pushString(char *)
{
	strcpy(&s_buf[s_bufidx], yytext);
	s_bufidx += strlen(yytext);
}

static bool flushLine(bool ret)
{
	int l = s_line;
	if (ret) { s_buf[s_bufidx++] = '\n'; s_line++;}
	s_buf[s_bufidx] = 0;

	/* \begin{} ܂\end{}̌̌Ăяo */
	if (blockNameProc) {
		const bool flush_first =
			(blockNameProc == endBlock) &&
			 (s_error != PHTexBlockParseListener::BPE_NO_CONVERSION)? true: false;

		/* ̍subNɊ܂߂ꍇ */
		if (flush_first) { s_listener->line(l, s_buf); }

		s_error = (*blockNameProc)(s_blockname);
		if (s_error != PHTexBlockParseListener::BPE_NOERROR &&
		    s_error != PHTexBlockParseListener::BPE_NO_CONVERSION ) {
			return false;
		}
		/* ̍subNɊ܂߂Ȃꍇ */
		if (!flush_first) { s_listener->line(l, s_buf); }
		blockNameProc = 0;

	} else {
		s_listener->line(l, s_buf);
	}

	s_bufidx = 0;
	return true;
}

static void chomp(char *str)
{
     int len = strlen(str);
     if (len > 0) {
	     char *p = &str[len-1];
	     while (*p == '\n' || *p == '\r') {
		     *p = 0; p--;
	     }
     }
}

void setListenerPHTexBlock(PHTexBlockParseListener *p) { s_listener = p; }

#include "ExceptionDef.h"

static char *s_error_fmt[] =
{
	"%d : no error",
	"%d : block name not match",
	"%d : block doesn't end before new block start",
	"%d : something wrong...",
	"%d : cannot convert equation",
};

bool tokenizePHTexBlock(FILE *fp)
{
	init();
        if (!s_listener) { assert(0); }
	yyin = fp;
	yylex();

	/* Ō̍sɉsȂꍇ̑Ώ */
	flushLine(false);

	if (s_error != PHTexBlockParseListener::BPE_NOERROR) {
		DTHROW_MESSAGE(RuntimeException,
			       (s_error_fmt[s_error], s_line+1));
	}
        return (!s_error)? true: false;
}

#ifdef TEXBLOCK_TEST

#include "PHTexBlockContext.h"

class Printer : public PHTexBlockParseListener
{
private:
	PHTexBlockContext::DefaultBlock *m_default;
	PHTexBlockContext m_context;
	bool  m_out;
        int   m_level;

public:
	Printer() : m_default(0), m_out(false), m_level(-1) {;}
private:
	Error startBlock(const char *name, int level)
	{
                if (m_out) {
			//                      printf("error : new block in equation block\n");
			return PHTexBlockParseListener::BPE_BLOCK_NOT_END;
                }
                       
                m_level = level;
		if (m_default) {
			m_context.push(m_default);
			m_default = 0;
		}
		fprintf(stderr, "block start : %s(%d)\n", name, level);
		m_out = m_context.convert(name);
		return BPE_NOERROR;
	}
	Error endBlock(const char *name, int level)
	{
                if (m_out && m_level != level) {
			//                      printf("error : wrong block depth(%d, %d)\n", m_level, level);
			return PHTexBlockParseListener:: BPE_WRONG_BLOCK_DEPTH;
                }
		fprintf(stderr, "block end : %s(%d)\n", name, level);
		m_out = false;
                m_level = -1;
		return BPE_NOERROR;
	}
	void line(int l, const char *str)
	{

                fprintf(stdout, "%d ##", l)		;
                fputs(str, stdout)			;
	}
	void defaultBlock(const char *name)
	{
		fprintf(stderr, "default block : \"%s\"\n", name);
		if (!m_default) { 
			m_default = new PHTexBlockContext::DefaultBlock;
		}
		m_default->pushBlock(name);
	}
	void defaultOption(const char *str)
	{
		fprintf(stderr, "default option : \"%s\"\n", str);
	}

	void convertBlock(int n)
	{
		m_context.push(new PHTexBlockContext::ConvertBlock(n));
		fprintf(stderr, "convert %d block\n", n);
	}

	void ignoreBlock(int n)
	{
		m_context.push(new PHTexBlockContext::IgnoreBlock(n));
		fprintf(stderr, "ignore next %d blocks\n", n);
	}

	void blockOption(const char *str)
	{
		fprintf(stderr, "option : \"%s\"\n", str);
	}
};

int main(int argc, char **argv)
{
	Printer printer;
	setListenerPHTexBlock(&printer);

	char *defaultfname = "./1.txm";
	char *fname = defaultfname;
	if (argc == 2) {
		fname = argv[1];
	}
	FILE *fp = fopen(fname, "r");
	if (!fp) { fprintf(stderr, "error\n"); return 1; }
	try {
		tokenizePHTexBlock(fp);
	} catch (Exception &e) {
		fprintf(stderr, "%s\n", e.msg());
	}
	fclose(fp);
	return 0;
}
#endif
