/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: main.cpp,v 1.22 2004/03/08 07:45:06 yoshi Exp $ */
#include "ExceptionDef.h"

#include "TextFileLineReadIterator.h"
#include "TextStdinLineReadIterator.h"
#include "FileTextWriter.h"
#include "StdoutTextWriter.h"

#include "PHTexArgsTokenizer.h"
#include "PHTexBlockConverter.h"

#include "Version.h"
#include "PHTexRC.h"
#include "BackSlashDesc.h"
#include <stdio.h>

#define ERR(MSG) fputs(MSG, stderr)

static int readMath2D(FILE *in, FILE *out, unsigned options)
{
	char buf[1024];
	while (fgets(buf, sizeof(buf), in) != 0) {
		PHTexPushLine(buf);
	}

	char *to = PHTexToTex(options);
	char *err = PHTexGetLastError();
	if (err) {
		ERR(err);
	}
	if (to) {
		fputs(to, out);
	}
	return (to)? 0: 1;
}

static int readTex(FILE *in, FILE *out, unsigned options)
{
	options &= ~PHTEX_DOCUMENT;
	PHTexBlockConverter convert;
	return convert(in, out, options)? 0: 1;
}

static void printTexDesc()
{
	printf("\nsupported Tex-like descriptions\n\n");

	int n;
	char **strs = getBackSlashDesc(&n);
	int	cnt = 0;
	for (int i=0; i<n; i++) {
		if (!strs[i]) { printf("\n"), cnt = 0; continue;}
		if (cnt%5 == 0) { printf("\n\t"); cnt = 0; }
		printf("%-8s ", strs[i]);
		cnt++;
	}
	printf("\n");
}

#define FREE(P) if (P) { delete P; P = 0; }

static void printVersion(char *progname,FILE *out)
{
	fprintf(out, "%s : %s\n", progname, getVersion());
}

int main(int argc,  char **argv)
{
	printVersion(argv[0], stderr);
	PHTexInit();
	int	ret = 1;
	PHTexArgsTokenizer tokenize;
	try { 
		tokenize(argc, argv);
		if (!tokenize.ignoreRC()) {
			int n;
			char **args = PHTexRCGetOptions(argv[0], &n);
			if (args) {
				tokenize(n, args);
			}
		}
	} catch (PHTexArgsTokenizer::PrintHelpException &) {
		char *appname = argv[0];
		tokenize.printHelp(appname);
		return 0;
	} catch (PHTexArgsTokenizer::PrintVersionException &) {
		return 0;
	} catch (PHTexArgsTokenizer::PrintTexLikeDescriptin &) {
		printTexDesc();
		return 0;
	} catch (PHTexArgsTokenizer::IllegalOptionException &) {
		ERR(tokenize.lastErr());
		return 1;
	} catch (Exception &e) {
		ERR(e.msg());
		return 1;
	}

	TextLineReadIteratorImpl *in = 0;
	TextWriterImpl *out = 0;

	char *infile = tokenize.infile();
	char *outfile = tokenize.outfile();

	DTRY {
		if (infile){
			in = new TextFileLineReadIterator(infile);
			fprintf(stderr, "reading %s ...\n", infile);
		} else {
			in = new TextStdinLineReadIterator;
		}

		if (outfile) {
			out = new FileTextWriter(outfile);
		} else {
			out = new StdoutTextWriter;
		}

		typedef int (ReadProc)(FILE *, FILE *, unsigned);
		const unsigned opts = tokenize.options();
		const unsigned v = tokenize.applyNegativeOptions(opts);
			
		ReadProc *proc = tokenize.inputTex()?
			readTex: readMath2D;
		ret = (*proc)(in->file(), out->file(), v);
	} DCATCH_START(Exception) {
		ERR(e.msg());
	} DCATCH_END();
 
	FREE(in);
	FREE(out);

	PHTexFree();

	return ret;
}
