/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: FileUtil.cpp,v 1.1 2003/10/06 11:14:50 yoshi Exp $ */ 
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#ifdef Linux
#include <unistd.h>
#elif WIN32
#include <windows.h>
#endif

#include "FileUtil.h"

#define SPLITER '/'

static char spliter_str[2] = { SPLITER, 0, };

int FileUtil::spliterLength()
{
	return 1;
}

char*	FileUtil::spliterStr() { return spliter_str; }

char *FileUtil::removeSpliter(char *dir)
{
	int n = strlen(dir);
	if (dir[n-1] == SPLITER) {
		dir[n-1] = 0;
	}
	return dir;
}


char *FileUtil::filename(const char *parent, const char *name, char *buf)
{
	char *p;
	p = strcpy(buf, parent);
	assert(p);
	if (buf[strlen(parent)-1] != SPLITER) {
		strcat(buf, spliter_str); assert(p);
	}
	strcat(buf, name); assert(p);
	return buf;
}

char *FileUtil::filenameHead(char *str)
{
	int n = strlen(str);
	char *head = str;
	char *tail = &str[n-1];
	for (char *p=tail; p != head; p--) {
		if (*p == SPLITER && *(p+1) != 0) return p+1;
	}
	return head;
}

bool FileUtil::move(const char *from, const char *to)
{
#ifdef Linux
	if (link(from, to) == -1) { return false; }
	if (unlink(from) == -1) { return false; }
	return true;
#elif WIN32
	return (MoveFile(from, to))? true: false;
#endif
}

bool FileUtil::remove(const char *fname)
{
#ifdef Linux
	return (unlink(fname) != -1)? true: false;
#elif WIN32
	return (_unlink(fname) == 0)? true: false;
#endif
}


static char* findSpliter(char *str)
{
	for (char *p=str; *p; p++) {
		if (*p == SPLITER) { return p; }
	}
	return 0;
}



bool FileUtil::recurMkdir(const char *dirname)
{
	static char buf[1024];
	static char spliter_str[2] = { SPLITER, 0, };
	strcpy(buf, dirname);
	strcat(buf, spliter_str);

	char *p = buf;
	while (*p == SPLITER) { p++;}  // ŏ'/'ƃ_
	while(p = findSpliter(p)) {
		*p = 0;
		if (!fileExist(buf)) { 
			if (!mkdir(buf)) { return false; }
		}
		*p = SPLITER;
		p++;
	}
	return true;
}

bool FileUtil::touch(const char *fname)
{
	FILE *fp = fopen(fname, "a");
	if (!fp) { return false; }
	fclose(fp);
	return true;
}


bool FileUtil::copy(const char *src, const char *to)
{
#ifdef WIN32
	BOOL b = ::CopyFile(src, to, FALSE);
	return (b)? true: false;
#else
	assert(0);
	return false;
#endif	

}
	
	


#ifdef FILEUTIL_TEST

#include <stdio.h>

#if 0
int main()
{
	const char *dir1 = "foo/var/buz1";
	const char *dir2 = "foo/var/buz2";

	bool b;
	b = FileUtil::recurMkdir(dir1);
	b = FileUtil::recurMkdir(dir2);

}
#endif

#define ERR(str) fprintf(stderr, "%s\n", str)

int main()
{
	const char *from = "foo";
	const char *to = "var";
	if (FileUtil::move(from, to)) { ERR("true"); }
	else { ERR("false"); }
	return 0;
}


#endif


